// start TRANS_DETECT first

public class BANDED_TRANS_SAMPLER{
    // signal chain
    adc => JCRev rev => Envelope e => LiSa loopme => dac;
    e => HPF lhp => LPF lp => LiSa low => dac;
    e => LPF bpl => HPF bph => LiSa mid => dac; // filter before recording?
    e => HPF hp => Gain gh => LiSa hi => dac;
    
    .005 => rev.mix;
    100 => lhp.freq;
    3 => lhp.Q;
    1000 => lp.freq;
    1000 => bph.freq;
    3500 => bpl.freq;
    3500 => hp.freq;
    3 => hp.Q;
    1.5 => gh.gain;
    
    CENTROID c;
    c.getUpdateRate() => dur updateRate;
    
    60::second => loopme.duration => low.duration => mid.duration => hi.duration;
    20::ms => dur fadetime;
    60::second => dur looplen;
    fadetime => e.duration;
    
    dur atTime[100];
    dur LowAtTime[100];
    dur MidAtTime[100];
    dur HiAtTime[100];
    0 => int numHits;
    0 => int numLowHits;
    0 => int numMidHits;
    0 => int numHiHits;
    
    int voiceLow;
    
    fun int detect(){
        if (TRANS_DETECT.last > .001 && TRANS_DETECT.slope > .01) // adjust these values as needed
            return 1;
        else
            return 0;
    }
    
    fun void playSomething(){
        // get a voice 
        //loopme.getVoice() => voiceLow;
        // set playhead
        loopme.playPos(atTime[Std.rand2(0,numHits)] - 10::ms);
        // play
        loopme.play(1);
        150::ms => now;
        // stop
        loopme.play(0);
        10::ms => now;
    }
    
    fun void playLow(){
        // get a voice 
        //loopme.getVoice() => voiceLow;
        // set playhead
        low.playPos(LowAtTime[Std.rand2(0,numLowHits)] - 10::ms);
        // play
        low.play(1);
        150::ms => now;
        // stop
        low.play(0);
        10::ms => now;
    }
    
    fun void playMid(){
        // set playhead
        mid.playPos(MidAtTime[Std.rand2(0,numMidHits)] - 10::ms);
        // play
        mid.play(1);
        150::ms => now;
        // stop
        mid.play(0);
        10::ms => now;
    }
    
    fun void playHi(){
        // set playhead
        hi.playPos(HiAtTime[Std.rand2(0,numHiHits)] - 10::ms);
        // play
        hi.play(1);
        50::ms => now;
        // stop
        hi.play(0);
        10::ms => now;
    }
    
    // var to check when the last hit was recorded
    1::second => dur prev;
    // make sure its been a 1/10th of a second to get a new hit
    50::ms => dur minGap;
    
    fun void recordSomething(dur length){
        if (length > looplen)
            looplen => length;
        // record
        <<< "recording" >>>;
        loopme.record(1);
        low.record(1);
        mid.record(1);
        hi.record(1);
        
        e.keyOn();
        now + (length - fadetime) => time later;
        
        while(now < later){
            
            if (detect() > 0){
                if (numHits > 0)
                    loopme.recPos()-atTime[numHits-1] => prev;
                if ((detect() > 0) && prev > minGap ){ 
                    loopme.recPos() => atTime[numHits];   
                    numHits++;
                    
                    if (c.centVal <= 1500){
                        <<< "low recorded at: ", loopme.recPos()/44100, "centroid:", c.centVal  >>>;
                        loopme.recPos() => LowAtTime[numLowHits]; 
                        numLowHits++;
                    }
                    else if (c.centVal > 1500 && c.centVal < 2500){
                        <<< "mid recorded at: ", loopme.recPos()/44100, "centroid:", c.centVal  >>>;
                        loopme.recPos() => MidAtTime[numMidHits];
                        numMidHits++;
                    }
                    else{
                        <<< "hi recorded at: ", loopme.recPos()/44100, "centroid:", c.centVal  >>>;
                        loopme.recPos() => HiAtTime[numHiHits]; 
                        numHiHits++;
                    }
                }
            }
            updateRate => now;
        }
        e.keyOff();
        // stop!
        <<< "done recording ">>>;
        loopme.record(0);
        low.record(0);
        mid.record(0);
        hi.record(0);
    }
}

