// start TRANS_DETECT first

public class PITCH_SAMPLER{
    // signal chain
    adc => Envelope e => LiSa pitchLoop => dac;
    
    60::second => pitchLoop.duration;
    20::ms => dur fadetime;
    30::second => dur looplen;
    fadetime => e.duration;
    
    int voice;
    
    100 => int maxHits;
    dur pitchAtTime[maxHits];
    int pitchDetected[maxHits];
    int patternPlayed[maxHits];
    0 => int pitchPtr;
    0 => int numHits;
    0 => int patternLength;
    0 => int mode;
    
    fun int detect(){
        if (TRANS_DETECT.last > .001 && TRANS_DETECT.slope > .01) // adjust these values as needed
            return 1;
        else
            return 0;
    }
    
    fun int getRoot(){
        int p[12];
        for (0 => int i; i < patternLength; i++){
            patternPlayed[i] % 12 => int note;
            p[note]++;
        }
        // find max:
        0 => int max;
        for(0 => int i; i < 12; i++){
            if (p[i] > max)
                i => max;
        }
        //<<< "max",  max >>>;
        return max;
    }
    
    fun void setMode(int newMode){
        newMode => mode;
        // reset pointer
        0 => pitchPtr;
    }
    
    fun void playSample(){
        // get a voice and assign it to the next in the array
        pitchLoop.getVoice() => voice;
        // set playhead
        Std.rand2(0,numHits) => int  index;
        pitchLoop.playPos(pitchAtTime[index]);
        
        pitchDetected[index] => patternPlayed[pitchPtr];
        pitchPtr++;
        pitchPtr => patternLength;
        // play
        pitchLoop.play(1);
        300::ms => now;
        // stop
        pitchLoop.play(0);
        10::ms => now;
    }
    
    fun int getNote(){
        patternPlayed[pitchPtr] => int note;
        pitchPtr++;
        patternLength %=> pitchPtr;
        return note;
    }
    
    fun int getMode(){
        return mode;
    }
    
    fun void recordSomething(dur length){
        if (length > looplen)
            looplen => length;
        // record
        <<< "pitch sampler recording" >>>;
        pitchLoop.record(1);
        
        e.keyOn();
        now + (length - fadetime) => time later;
        
        1::second => dur prev;
        
        while(now < later){
            if (PITCH_ESTIMATE.max > PITCH_ESTIMATE.threshold){        
                if (numHits > 0)
                    pitchLoop.recPos()-pitchAtTime[numHits-1] => prev;
                if ((detect() > 0) && (prev > .1*44100::samp && numHits < maxHits) ){ // make sure its been a 1/10th of a second to get a new hit
                <<< "pitch recorded at: ", pitchLoop.recPos()/44100 >>>;
                pitchLoop.recPos() - 30::ms => pitchAtTime[numHits];   
                PITCH_ESTIMATE.freqEst => Std.ftom => float est;
                est$int => pitchDetected[numHits];
                numHits++;
            }
        }
        10::ms => now;
        }
        e.keyOff();
        // stop!
        pitchLoop.record(0);
    }
    

    //spork ~ playSomething(vPtr, atTime[Std.rand2(0,numHits)]);

}

